@@ -26,6 +26,11 @@ SEXP readfile();
 SEXP reorder();
 SEXP rorder_tol();
 SEXP rbindlist();
+SEXP EvalDepth();
+SEXP vecseq();
+SEXP copyattr();
+SEXP setlistelt();
+SEXP setnamed();
 
 // .Externals
 SEXP fastmean();
@@ -51,6 +56,11 @@ R_CallMethodDef callMethods[] = {
 {"Creorder", (DL_FUNC) &reorder, -1},
 {"Crorder_tol", (DL_FUNC) &rorder_tol, -1},
 {"Crbindlist", (DL_FUNC) &rbindlist, -1},
+{"CEvalDepth", (DL_FUNC) &EvalDepth, -1},
+{"Cvecseq", (DL_FUNC) &vecseq, -1},
+{"Ccopyattr", (DL_FUNC) &copyattr, -1},
+{"Csetlistelt", (DL_FUNC) &setlistelt, -1},
+{"Csetnamed", (DL_FUNC) &setnamed, -1},
 {NULL, NULL, 0}
 };
 
